@@ -897,28 +897,26 @@ ngx_filename_cmp(u_char *s1, u_char *s2, size_t n)
 ngx_int_t
 ngx_atoi(u_char *line, size_t n)
 {
-    ngx_int_t  value, cutoff, cutlim;
+    ngx_int_t  value;
 
     if (n == 0) {
         return NGX_ERROR;
     }
 
-    cutoff = NGX_MAX_INT_T_VALUE / 10;
-    cutlim = NGX_MAX_INT_T_VALUE % 10;
-
     for (value = 0; n--; line++) {
         if (*line < '0' || *line > '9') {
             return NGX_ERROR;
         }
 
-        if (value >= cutoff && (value > cutoff || *line - '0' > cutlim)) {
-            return NGX_ERROR;
-        }
-
         value = value * 10 + (*line - '0');
     }
 
-    return value;
+    if (value < 0) {
+        return NGX_ERROR;
+
+    } else {
+        return value;
+    }
 }
 
 
@@ -927,16 +925,13 @@ ngx_atoi(u_char *line, size_t n)
 ngx_int_t
 ngx_atofp(u_char *line, size_t n, size_t point)
 {
-    ngx_int_t   value, cutoff, cutlim;
+    ngx_int_t   value;
     ngx_uint_t  dot;
 
     if (n == 0) {
         return NGX_ERROR;
     }
 
-    cutoff = NGX_MAX_INT_T_VALUE / 10;
-    cutlim = NGX_MAX_INT_T_VALUE % 10;
-
     dot = 0;
 
     for (value = 0; n--; line++) {
@@ -958,107 +953,98 @@ ngx_atofp(u_char *line, size_t n, size_t point)
             return NGX_ERROR;
         }
 
-        if (value >= cutoff && (value > cutoff || *line - '0' > cutlim)) {
-            return NGX_ERROR;
-        }
-
         value = value * 10 + (*line - '0');
         point -= dot;
     }
 
     while (point--) {
-        if (value > cutoff) {
-            return NGX_ERROR;
-        }
-
         value = value * 10;
     }
 
-    return value;
+    if (value < 0) {
+        return NGX_ERROR;
+
+    } else {
+        return value;
+    }
 }
 
 
 ssize_t
 ngx_atosz(u_char *line, size_t n)
 {
-    ssize_t  value, cutoff, cutlim;
+    ssize_t  value;
 
     if (n == 0) {
         return NGX_ERROR;
     }
 
-    cutoff = NGX_MAX_SIZE_T_VALUE / 10;
-    cutlim = NGX_MAX_SIZE_T_VALUE % 10;
-
     for (value = 0; n--; line++) {
         if (*line < '0' || *line > '9') {
             return NGX_ERROR;
         }
 
-        if (value >= cutoff && (value > cutoff || *line - '0' > cutlim)) {
-            return NGX_ERROR;
-        }
-
         value = value * 10 + (*line - '0');
     }
 
-    return value;
+    if (value < 0) {
+        return NGX_ERROR;
+
+    } else {
+        return value;
+    }
 }
 
 
 off_t
 ngx_atoof(u_char *line, size_t n)
 {
-    off_t  value, cutoff, cutlim;
+    off_t  value;
 
     if (n == 0) {
         return NGX_ERROR;
     }
 
-    cutoff = NGX_MAX_OFF_T_VALUE / 10;
-    cutlim = NGX_MAX_OFF_T_VALUE % 10;
-
     for (value = 0; n--; line++) {
         if (*line < '0' || *line > '9') {
             return NGX_ERROR;
         }
 
-        if (value >= cutoff && (value > cutoff || *line - '0' > cutlim)) {
-            return NGX_ERROR;
-        }
-
         value = value * 10 + (*line - '0');
     }
 
-    return value;
+    if (value < 0) {
+        return NGX_ERROR;
+
+    } else {
+        return value;
+    }
 }
 
 
 time_t
 ngx_atotm(u_char *line, size_t n)
 {
-    time_t  value, cutoff, cutlim;
+    time_t  value;
 
     if (n == 0) {
         return NGX_ERROR;
     }
 
-    cutoff = NGX_MAX_TIME_T_VALUE / 10;
-    cutlim = NGX_MAX_TIME_T_VALUE % 10;
-
     for (value = 0; n--; line++) {
         if (*line < '0' || *line > '9') {
             return NGX_ERROR;
         }
 
-        if (value >= cutoff && (value > cutoff || *line - '0' > cutlim)) {
-            return NGX_ERROR;
-        }
-
         value = value * 10 + (*line - '0');
     }
 
-    return value;
+    if (value < 0) {
+        return NGX_ERROR;
+
+    } else {
+        return value;
+    }
 }
 
 
@@ -1066,19 +1052,13 @@ ngx_int_t
 ngx_hextoi(u_char *line, size_t n)
 {
     u_char     c, ch;
-    ngx_int_t  value, cutoff;
+    ngx_int_t  value;
 
     if (n == 0) {
         return NGX_ERROR;
     }
 
-    cutoff = NGX_MAX_INT_T_VALUE / 16;
-
     for (value = 0; n--; line++) {
-        if (value > cutoff) {
-            return NGX_ERROR;
-        }
-
         ch = *line;
 
         if (ch >= '0' && ch <= '9') {
@@ -1096,7 +1076,12 @@ ngx_hextoi(u_char *line, size_t n)
         return NGX_ERROR;
     }
 
-    return value;
+    if (value < 0) {
+        return NGX_ERROR;
+
+    } else {
+        return value;
+    }
 }
 
 
